/* RPC extension for IP netfilter matching, Version 2.2
 * (C) 2000 by Marcelo Barbosa Lima <marcelo.lima@dcc.unicamp.br>
 *      - original rpc tracking module
 *      - "recent" connection handling for kernel 2.3+ netfilter
 *
 * (C) 2001 by Rusty Russell <rusty@rustcorp.com.au>
 *      - upgraded conntrack modules to oldnat api - kernel 2.4.0+
 *
 * (C) 2002 by Ian (Larry) Latter <Ian.Latter@mq.edu.au>
 *      - upgraded conntrack modules to newnat api - kernel 2.4.20+
 *      - extended matching to support filtering on procedures
 *
 * (c) 2008 by Franz Regensburger (FABR) <franz.regensburger@odorf.de>
 *      - added structure ip_ct_rpc_master for helper private data (conntrack master)
 *        in order to support the
 *        - rpc program 100000 (portmapper) procedure DUMP
 *        Currently, we did not register our helper structure (struct ip_ct_rpc_master)
 *        in union ip_conntrack_help (file ip_conntrack.h).
 *        However, we check whether our helper structure needs less or equal size than
 *        the officialy registered helper sturcture ip_ct_ftp_master.
 *
 *        ip_ct_ftp_master: (4+4)*2 bytes
 *        ip_ct_rpc_master: (4+2)*2 bytes
 *
 * ipt_rpc.h.c,v 2.2 2003/01/12 18:30:00
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 **
 */

#ifndef _IPT_RPC_H
#define _IPT_RPC_H

struct ipt_rpc_data;

struct ipt_rpc_info {
    int inverse;
    int strict;
    const char c_procs[1408];
    /* FABR: documentation for the value 1408 (took me some time to figure it out).
     * The file iptables-.../extensions/libipt_rpc.c defines the following 2 constants
     *   const int IPT_RPC_MAX_ENTS = 128
     *   const int IPT_RPC_CHAR_LEN = 11
     * They are used by the parser for the rule syntax of the module ipt_rpc.c.
     * You may specify a maximum of 128 RPC programs in the rule.
     * An RPC program is a u_int_32 decimal number which yields a maximal value of
     * 2^32 -1 = 4294967296 (decimal). The ASCII represenation of the largest RPC program
     * possible needs 10 characters. Therefore, the parsed string needs at most 11 bytes.
     * Hence: 128 * 11 = 1408
     *
     */
    int i_procs;
    struct ipt_rpc_data *data;
};

struct ip_ct_rpc_master {
    u_int32_t next_seq_num[IP_CT_DIR_MAX];
    u_int16_t bitfield[IP_CT_DIR_MAX];
};

/* define bit masks and flags for the bitfield in struct ip_ct_rpc_master */

/* nibble 1 */
/* main control flags for tracking RM fragments of a tcp stream */
#define RPC_CT_FLAGS_VALID_MASK          0x8000      /* L000 */
#define RPC_CT_FLAGS_VALID_NO            0x0000      /* 0000 */
#define RPC_CT_FLAGS_VALID_YES           0x8000      /* L000 */

#define RPC_CT_FLAGS_INITIAL_RM_MASK     0x4000      /* 0L00 */
#define RPC_CT_FLAGS_INITIAL_RM_NO       0x0000      /* 0000 */
#define RPC_CT_FLAGS_INITIAL_RM_YES      0x4000      /* 0L00 */

/* the fate of a truncated map entry */
#define RPC_CT_FLAGS_MAPFATE_MASK        0x3000      /* 00LL */
#define RPC_CT_FLAGS_MAPFATE_UNDEF       0x0000      /* 0000 */
#define RPC_CT_FLAGS_MAPFATE_UNKNOWN     0x1000      /* 000L */
#define RPC_CT_FLAGS_MAPFATE_DROP        0x2000      /* 00L0 */
#define RPC_CT_FLAGS_MAPFATE_PASS        0x3000      /* 00LL */

/* nibble 2: the protocol of the truncated map entry */
#define RPC_CT_FLAGS_PROTO_MASK          0x0F00      /* LLLL */
#define RPC_CT_FLAGS_PROTO_UNDEF         0x0000      /* 0000 */
#define RPC_CT_FLAGS_PROTO_TCP           0x0100      /* 000L */
#define RPC_CT_FLAGS_PROTO_UDP           0x0200      /* 00L0 */

/* nibble 3+4: the number of missing bytes in a truncated map entry */
#define RPC_CT_FLAGS_MISSING_BYTES_MASK  0x00FF    /* LLLL LLLL */

#endif /* _IPT_RPC_H */
